@@ -22,7 +22,7 @@
 # TODO:
 #  - better annotate algorithm on packs
 
-import sys, time
+import sys, time, md5
 from itertools import groupby
 from PyQt4 import QtCore, QtGui
 
@@ -54,6 +54,7 @@ from bzrlib.plugins.qbzr.lib.texteditannotate import (AnnotateBarBase,
                                                       AnnotateEditerFrameBase)
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), '''
+from bzrlib.config import parse_username
 from bzrlib.workingtree import WorkingTree
 from bzrlib.revisiontree import RevisionTree
 from bzrlib.plugins.qbzr.lib.revisionmessagebrowser import LogListRevisionMessageBrowser
@@ -266,12 +267,14 @@ class AnnotateWindow(QBzrWindow):
 
     def __init__(self, branch, working_tree, annotate_tree, path, fileId,
                  encoding=None, parent=None, ui_mode=True, no_graph=False,
-                 loader=None, loader_args=None):
+                 loader=None, loader_args=None, activate_line=None):
         QBzrWindow.__init__(self,
                             [gettext("Annotate"), gettext("Loading...")],
                             parent, ui_mode=ui_mode)
         self.restoreSize("annotate", (780, 680))
 
+        self.activate_line_after_load = activate_line
+
         self.windows = []
 
         self.branch = branch
@@ -417,6 +420,8 @@ class AnnotateWindow(QBzrWindow):
                      QtCore.SIGNAL("toggled (bool)"),
                      self.show_goto_line_toggle )
 
+        self.__hashes = {}
+
     def show(self):
         QBzrWindow.show(self)
         QtCore.QTimer.singleShot(0, self.initial_load)
@@ -448,7 +453,9 @@ class AnnotateWindow(QBzrWindow):
                 self.branch.unlock()
         finally:
             self.throbber.hide()
-    
+        if self.activate_line_after_load:
+            self.go_to_line(self.activate_line_after_load)
+
     def set_annotate_title(self):
         # and update the title to show we are done.
         if isinstance(self.annotate_tree, RevisionTree):
@@ -474,8 +481,7 @@ class AnnotateWindow(QBzrWindow):
         lines = []
         annotate = []
         ordered_revids = []
-        
-        
+
         self.processEvents()
         for revid, text in annotate_tree.annotate_iter(fileId):
             if revid == CURRENT_REVISION:
@@ -591,29 +597,56 @@ class AnnotateWindow(QBzrWindow):
                 old_char_start += old_len
                 new_char_start += new_len
         return new_positions
-    
+
     def revisions_loaded(self, revisions, last_call):
         for rev in revisions.itervalues():
-            author_name = get_apparent_author_name(rev)
-            
+            authors = rev.get_apparent_authors()
+            emails = map(self._maybe_extract_email, authors)
+            author_id = ';'.join(emails)
+
             if rev.timestamp is None:
                 days = sys.maxint
             elif self.now < rev.timestamp:
                 days = 0
             else:
                 days = (self.now - rev.timestamp) / (24 * 60 * 60)
-            
+
             alpha = 0.5/((days/50) + 1)
-            hue =  1-float(abs(hash(author_name))) / sys.maxint 
+            h_sh = self._get_hash(author_id)
+            hue =  1-float(h_sh) / sys.maxint
             color = QtGui.QColor.fromHsvF(hue, 1, 1, alpha)
             brush = QtGui.QBrush(color)
-            
+
             self.annotate_bar.rev_colors[rev.revision_id] = brush
             self.text_edit.rev_colors[rev.revision_id] = brush
-        
+
         self.annotate_bar.update()
         self.text_edit.update()
 
+    def _maybe_extract_email(self, author):
+        name, email = parse_username(author)
+        if email:
+            return email
+        return author
+
+    def _get_hash(self, author_id):
+        h = self.__hashes.get(author_id)
+        if h is None:
+            h = abs(hash(author_id))
+            #h = abs(hash(md5.md5(author_id).hexdigest())) # maybe increase enthropy via md5?
+            # XXX [bialix 2012/03/30]
+            # I don't really like how we create colors for annotation.
+            # Also, abs(hash) loses 1 bit of the hash result.
+            # I think we can improve our algorithm to use more distinctive colors
+            # maybe we should use md5 as more enthropy source for hash,
+            # or maybe we can use some sort of predefined set of colors
+            # and each next author will use color from that set,
+            # but in this case we lose consistency of colors for the same author
+            # between different annotations
+            # I have no clever idea yet.
+            self.__hashes[author_id] = h
+        return h
+
     def edit_cursorPositionChanged(self):
         current_line = self.text_edit.document().findBlock(
             self.text_edit.textCursor().position()).blockNumber()
@@ -684,6 +717,14 @@ class AnnotateWindow(QBzrWindow):
         else:
             self.text_edit.setLineWrapMode(QtGui.QPlainTextEdit.NoWrap)
 
+    def go_to_line(self, line):
+        doc = self.text_edit.document()
+        cursor = QtGui.QTextCursor(doc)
+        cursor.setPosition(doc.findBlockByNumber(line-1).position())
+        self.text_edit.setTextCursor(cursor)
+        self.text_edit.centerCursor()
+
+
 # QIntValidator did not work on vila's setup, so this is a workaround.
 class IntValidator(QtGui.QValidator):
     def validate (self, input, pos):
@@ -698,6 +739,7 @@ class IntValidator(QtGui.QValidator):
         else:
             return (QtGui.QValidator.Invalid, pos)
 
+
 class GotoLineToolbar(QtGui.QToolBar):
     
     def __init__(self, anotate_window, show_action):
@@ -745,19 +787,14 @@ class GotoLineToolbar(QtGui.QToolBar):
     
     def go_triggered(self, state=True):
         try:
-            line = int(str(self.line_edit.text()))-1
+            line = int(str(self.line_edit.text()))
         except ValueError:
             pass
         else:
-            doc = self.anotate_window.text_edit.document()
-            cursor = QtGui.QTextCursor(doc)
-            cursor.setPosition(doc.findBlockByNumber(line).position())
-            self.anotate_window.text_edit.setTextCursor(cursor)
-            self.anotate_window.text_edit.centerCursor()
+            self.anotate_window.go_to_line(line)
             self.show_action.setChecked(False)
 
 
-
 class AnnotateLogList(LogList):
     
     parent_annotate_window = None
