@@ -19,9 +19,13 @@
 
 from PyQt4 import QtCore, QtGui
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), '''
 from bzrlib.plugins.qbzr.lib.util import run_in_loading_queue
 from bzrlib.plugins.qbzr.lib.lazycachedrevloader import load_revisions
+from bzrlib.plugins.qbzr.lib.diff import ExtDiffContext
 from bzrlib.transport.local import LocalTransport
+''')
 
 RevIdRole = QtCore.Qt.UserRole + 1
 
@@ -45,15 +49,17 @@ class RevisionTreeView(QtGui.QTreeView):
             
         self.connect(self,
                      QtCore.SIGNAL("collapsed (QModelIndex)"),
-                     self.colapsed_expanded)
+                     self.collapsed_expanded)
         
         self.connect(self,
                      QtCore.SIGNAL("expanded (QModelIndex)"),
-                     self.colapsed_expanded)
+                     self.collapsed_expanded)
         
         self.load_revisions_call_count = 0
         self.load_revisions_throbber_shown = False
-    
+        self.revision_loading_disabled = False
+        self.diff_context = ExtDiffContext(self)
+
     def setModel(self, model):
         QtGui.QTreeView.setModel(self, model)
         
@@ -65,6 +71,9 @@ class RevisionTreeView(QtGui.QTreeView):
         model.connect(model,
                       QtCore.SIGNAL("dataChanged(QModelIndex, QModelIndex)"),
                       self.data_changed)
+        model.connect(model,
+                      QtCore.SIGNAL("layoutChanged()"),
+                      self.layout_changed)
     
     def scroll_changed(self, value):
         self.load_visible_revisions()
@@ -72,7 +81,10 @@ class RevisionTreeView(QtGui.QTreeView):
     def data_changed(self, start_index, end_index):
         self.load_visible_revisions()
     
-    def colapsed_expanded(self, index):
+    def layout_changed(self):
+        self.load_visible_revisions()
+    
+    def collapsed_expanded(self, index):
         self.load_visible_revisions()
     
     def resizeEvent(self, e):
@@ -80,7 +92,8 @@ class RevisionTreeView(QtGui.QTreeView):
         QtGui.QTreeView.resizeEvent(self, e)
     
     def load_visible_revisions(self):
-        run_in_loading_queue(self._load_visible_revisions)
+        if not self.revision_loading_disabled:
+            run_in_loading_queue(self._load_visible_revisions)
     
     def _load_visible_revisions(self):
         model = self.model()
@@ -93,11 +106,13 @@ class RevisionTreeView(QtGui.QTreeView):
         #    throbber_height = self.throbber.   etc...        
         bottom_index = self.indexAt(self.viewport().rect().bottomLeft()) # + throbber_height
         
-        revids = set()
+        revids = []
         while True:
             revid = index.data(RevIdRole)
             if not revid.isNull():
-                revids.add(str(revid.toByteArray()))
+                revid = str(revid.toByteArray())
+                if revid not in revids:
+                    revids.append(revid)
             if index == bottom_index:
                 break
             index = self.indexBelow(index)
@@ -164,7 +179,7 @@ def get_text_color ( option, style):
     
     if option.state & QtGui.QStyle.State_Selected:
         if has_vista_style and isinstance(style, QtGui.QWindowsVistaStyle):
-            # QWindowsVistaStyle normaly modifies it palette,
+            # QWindowsVistaStyle normally modifies it palette,
             # but as we can't reuse that code, we have to reproduce
             # what it does here.
             # https://bugs.edge.launchpad.net/qbzr/+bug/457895
